@@ -11,105 +11,8 @@ use libadwaita as adw;
 
 use crate::print_perf;
 
-/// An Action describes a mutation of the global state.
-///
-/// It has a name which is used for pattern matching and a payload.
-/// Since gstore is for the GTK framework these actions are linked
-/// to GTK Actions. GTK Actions use their own 'generic value type'
-/// called Variant. Thus gstore Actions have a Variant as argument.
-///
-/// # Examples
-///
-/// ```rust,ignore
-/// // given we receive an action `action` in a reducer
-/// // Note: Action::new is not public
-/// let action = Action::new("test".into(), Some(128_i32.to_variant()))
-/// // get a rust value from it by using
-/// if let Some(number) = action.arg::<i32>() {
-///     // do stuff
-/// }
-/// ```
-#[derive(Debug, Clone)]
-pub struct Action {
-    name: String,
-    #[cfg(feature = "latest-gtk")]
-    argument: Option<glib::Variant>,
-}
-
-impl Action {
-    /// Create a new action.
-    ///
-    /// **NOTE:**
-    ///
-    /// **This is a placeholder since rustdoc server can not compile latest gtk yet
-    /// The actual signature looks like this:**
-    /// ```rust
-    /// fn new(name: String, argument: Option<glib::Variant>) -> Self
-    /// ```
-    #[cfg(not(feature = "latest-gtk"))]
-    fn new(name: String, argument: Option<String>) -> Self {
-        Action { name }
-    }
-
-    /// Create a new action.
-    #[cfg(feature = "latest-gtk")]
-    fn new(name: String, argument: Option<glib::Variant>) -> Self {
-        Action { name, argument }
-    }
-
-    /// Get the name of the action
-    pub fn name(&self) -> &str {
-        &self.name
-    }
-
-    /// Create a new action.
-    ///
-    /// **NOTE:**
-    ///
-    /// **This is a placeholder since rustdoc server can not compile latest gtk yet
-    /// The actual signature looks like this:**
-    /// ```rust
-    /// pub fn argument(&self) -> Option<&glib::Variant>
-    /// ```
-    #[cfg(not(feature = "latest-gtk"))]
-    pub fn argument(&self) -> Option<String> {
-        None
-    }
-
-    /// Get the argument as glib::Variant.
-    #[cfg(feature = "latest-gtk")]
-    pub fn argument(&self) -> Option<&glib::Variant> {
-        self.argument.as_ref()
-    }
-
-    /// Try to get the argument as <T>. If either there is no argument
-    /// or the argument is of a different type, the returned Option
-    /// will be None.
-    ///
-    /// **NOTE:**
-    ///
-    /// **This is a placeholder since rustdoc server can not compile latest gtk yet
-    /// The actual signature looks like this:**
-    /// ```rust
-    /// pub fn arg<T: glib::FromVariant + Sized>(&self) -> Option<T>
-    /// ```
-    #[cfg(not(feature = "latest-gtk"))]
-    pub fn arg<T: Sized>(&self) -> Option<String> {
-        None
-    }
-
-    /// Try to get the argument as <T>. If either there is no argument
-    /// or the argument is of a different type, the returned Option
-    /// will be None.
-    #[cfg(feature = "latest-gtk")]
-    pub fn arg<T: glib::FromVariant + Sized>(&self) -> Option<T> {
-        if self.argument().is_some() {
-            self.argument().unwrap().get()
-        } else {
-            panic!("Action has no argument");
-        }
-    }
-}
+mod action;
+pub use action::*;
 
 struct Selector<S: std::fmt::Debug + Clone + Default + 'static> {
     name: String,
@@ -450,7 +353,7 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
 
                 print_perf!(
                     reducer(&action, state);
-                    format!("Reduce action {:?}", action.name)
+                    format!("Reduce action {:?}", action.name())
                 );
                 trace!("Reduced to: {:?}", state);
 
@@ -472,7 +375,7 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
                             }
                         }
                     };
-                    format!("Determine n of {} selectors for action {:?}", selectors.len(), action.name)
+                    format!("Determine n of {} selectors for action {:?}", selectors.len(), action.name())
                 );
 
                 let state_copy_for_middleware = state_lock.clone();
